import { NotExistAction, } from "./core";
import { createError, isFileSystemError, NoModificationAllowedError, NotFoundError, NotReadableError, TypeMismatchError, } from "./errors";
import { getParentPath } from "./util";
export class AbstractEntry {
    constructor(fs, path) {
        this.fs = fs;
        this.path = path;
        this.cp = (to, options, errors) => this.copy(to, options, errors);
        this.del = (options, errors) => this.delete(options, errors);
        this.mv = (to, options, errors) => this.move(to, options, errors);
        this.patch = (props, options, errors) => this.fs.patch(this.path, props, options, errors);
        this.remove = (options, errors) => this.delete(options, errors);
        this.rm = (options, errors) => this.delete(options, errors);
        this.toString = () => `${this.fs.repository}:${this.path}`;
    }
    async copy(to, options, errors) {
        options = { ...this.fs.defaultCopyOptions, ...options };
        return await this._copy(to, options, errors);
    }
    async delete(options, errors) {
        options = { ...this.fs.defaultDeleteOptions, ...options };
        try {
            const result = await this.__delete(options, errors);
            await this._afterDelete(result, options);
            return result;
        }
        catch (e) {
            const opts = options;
            await this._handleNoModificationAllowedError({ e }, errors, async (error) => {
                await this._afterDelete(false, opts, error);
            });
            return false;
        }
        finally {
            this.stats = undefined;
        }
    }
    getParent() {
        const parentPath = getParentPath(this.path);
        return this.fs.getDirectory(parentPath);
    }
    async move(to, options, errors) {
        let result = await this._copy(to, {
            ...this.fs.defaultMoveOptions,
            ...options,
            recursive: true,
        }, errors);
        if (!result) {
            return false;
        }
        result = await this.delete({
            ...this.fs.defaultDeleteOptions,
            ...options,
            recursive: true,
        }, errors);
        return result;
    }
    async stat(options, errors) {
        return await this.head(options, errors);
    }
    async toURL(options, errors) {
        return await this.fs.getURL(this.path, options, errors);
    }
    async __delete(options, errors) {
        const result = await this._beforeDelete(options);
        if (result != null) {
            return result;
        }
        try {
            await this._validate(options);
            const result = await this._deleteExisting(options, errors);
            return result;
        }
        catch (e) {
            if (isFileSystemError(e) && e.name === NotFoundError.name) {
                if (options.onNotExist === NotExistAction.Error) {
                    throw e;
                }
                return false;
            }
            else {
                throw e;
            }
        }
    }
    async _afterDelete(result, options, error) {
        const fs = this.fs;
        const afterDelete = fs.options.hook?.afterDelete;
        if (afterDelete && !options.ignoreHook) {
            await afterDelete(fs.repository, this.path, options, result, error);
        }
    }
    async _beforeDelete(options) {
        const fs = this.fs;
        const beforeDelete = fs.options.hook?.beforeDelete;
        if (beforeDelete && !options.ignoreHook) {
            return await beforeDelete(fs.repository, this.path, options);
        }
        return null;
    }
    _createError(error) {
        return createError({
            ...error,
            repository: this.fs.repository,
            path: this.path,
        });
    }
    _createNoModificationAllowedError(additional) {
        return this._createError({ ...NoModificationAllowedError, ...additional });
    }
    _createNotFoundError(additional) {
        return this._createError({ ...NotFoundError, ...additional });
    }
    _createNotReadableError(additional) {
        return this._createError({ ...NotReadableError, ...additional });
    }
    _createTypeMismatchError(additional) {
        return this._createError({ ...TypeMismatchError, ...additional });
    }
    async _handleNoModificationAllowedError(cause, errors, callback) {
        const error = this._createNoModificationAllowedError(cause);
        return await this.fs._handleFileSystemError(error, errors, callback);
    }
    async _handleNotFoundError(cause, errors, callback) {
        const error = this._createNotFoundError(cause);
        return await this.fs._handleFileSystemError(error, errors, callback);
    }
    async _handleNotReadableError(cause, errors, callback) {
        const error = this._createNotReadableError(cause);
        return await this.fs._handleFileSystemError(error, errors, callback);
    }
    async _handleTypeMismatchError(cause, errors, callback) {
        const error = this._createTypeMismatchError(cause);
        return await this.fs._handleFileSystemError(error, errors, callback);
    }
}
//# sourceMappingURL=AbstractEntry.js.map