import { DEFAULT_BUFFER_SIZE } from "univ-conv";
import { AbstractFile } from "./AbstractFile";
import { EntryType, ExistsAction, NoParentAction, NotExistAction, } from "./core";
import { createError, isFileSystemError, NoModificationAllowedError, NotFoundError, NotReadableError, SyntaxError, TypeMismatchError, } from "./errors";
import { INVALID_CHARS, normalizePath } from "./util";
if (!Promise.allSettled) {
    Promise.allSettled = (promises) => Promise.all(promises.map((p) => p
        .then((value) => ({
        status: "fulfilled",
        value,
    }))
        .catch((reason) => ({
        status: "rejected",
        reason,
    }))));
}
export class AbstractFileSystem {
    constructor(repository, options = {}) {
        this.repository = repository;
        this.options = options;
        this.cp = (fromPath, toPath, options, errors) => this.copy(fromPath, toPath, options, errors);
        this.del = (path, options, errors) => this.delete(path, options, errors);
        this.mkdir = (path, options, errors) => this.mkcol(path, options, errors);
        this.mv = (fromPath, toPath, options, errors) => this.move(fromPath, toPath, options, errors);
        this.rm = (path, options, errors) => this.delete(path, options, errors);
        this.unlink = (path, options) => this.delete(path, options);
        this.defaultReadOptions = {
            bufferSize: DEFAULT_BUFFER_SIZE,
            ignoreHook: false,
            ...options.defaultReadOptions,
        };
        this.defaultWriteOptions = {
            bufferSize: DEFAULT_BUFFER_SIZE,
            append: false,
            onExists: ExistsAction.Overwrite,
            onNotExist: NotExistAction.Ignore,
            ignoreHook: false,
            ...options.defaultWriteOptions,
        };
        this.defaultHeadOptions = {
            ignoreHook: false,
            ...options.defaultHeadOptions,
        };
        this.defaultPatchOptions = {
            ignoreHook: false,
            ...options.defaultPatchOptions,
        };
        this.defaultMkcolOptions = {
            onExists: ExistsAction.Error,
            onNoParent: NoParentAction.Error,
            ignoreHook: false,
            ...options.defaultMkdirOptions,
        };
        this.defaultDeleteOptions = {
            onNotExist: NotExistAction.Error,
            recursive: false,
            ignoreHook: false,
            ...options.defaultDeleteOptions,
        };
        this.defaultMoveOptions = {
            bufferSize: DEFAULT_BUFFER_SIZE,
            append: false,
            onExists: ExistsAction.Error,
            onNotExist: NotExistAction.Ignore,
            onNoParent: NoParentAction.Error,
            ignoreHook: false,
            ...options.defaultMoveOptions,
        };
        this.defaultCopyOptions = {
            bufferSize: DEFAULT_BUFFER_SIZE,
            append: false,
            onExists: ExistsAction.Error,
            onNotExist: NotExistAction.Ignore,
            onNoParent: NoParentAction.Error,
            recursive: false,
            ignoreHook: false,
            ...options.defaultCopyOptions,
        };
    }
    _createError(params) {
        return createError({ ...params, repository: this.repository });
    }
    async _handleError(params, errors, callback) {
        const error = this._createError(params);
        await this._handleFileSystemError(error, errors, callback);
    }
    async _handleFileSystemError(error, errors, callback) {
        if (callback) {
            await callback(error);
        }
        if (errors) {
            errors.push(error);
            return;
        }
        else {
            throw error;
        }
    }
    async copy(fromPath, toPath, options, errors) {
        options = { ...this.defaultCopyOptions, ...options };
        let info;
        try {
            info = await this._prepareCopy(fromPath, toPath, options);
        }
        catch (e) {
            if (errors) {
                errors.push(e);
                return false;
            }
            throw e;
        }
        const { from, to } = info;
        return await from.copy(to, options, errors);
    }
    async delete(path, options, errors) {
        const entry = await this.getEntry(path, options, errors);
        if (!entry) {
            return false;
        }
        return await entry.delete(options);
    }
    async dir(path, options, errors) {
        return await this.list(path, options, errors);
    }
    getDirectory(path) {
        const checked = this._checkPath(path);
        return this._doGetDirectory(checked);
    }
    async getEntry(path, options) {
        options = { ...options };
        if (path.endsWith("/")) {
            if (!options.type) {
                options.type = EntryType.Directory;
            }
        }
        if (options.type === EntryType.File) {
            return this.getFile(path);
        }
        if (options.type === EntryType.Directory) {
            return this.getDirectory(path);
        }
        const stats = await this.head(path, options);
        const entry = stats.size != null ? this.getFile(path) : this.getDirectory(path);
        entry.stats = stats;
        return entry;
    }
    getFile(path) {
        if (path.endsWith("/")) {
            throw createError({
                name: SyntaxError.name,
                repository: this.repository,
                path,
                message: `"${path}" seems to be a directory.`,
            });
        }
        const checked = this._checkPath(path);
        return this._doGetFile(checked);
    }
    async getURL(path, options, errors) {
        options = { method: "GET", ...options };
        const stats = await this.head(path, options, errors);
        if (stats == null) {
            return null;
        }
        return await this._doGetURL(path, stats.size == null, options);
    }
    async hash(path, options, errors) {
        const file = this.getFile(path);
        if (file == null) {
            return null;
        }
        return await file.hash(options, errors);
    }
    async head(path, options, errors) {
        options = { ...this.defaultHeadOptions, ...options };
        if (!options.type) {
            if (path.endsWith("/")) {
                options.type = EntryType.Directory;
            }
        }
        path = this._checkPath(path);
        try {
            let stats = await this._beforeHead(path, options);
            if (stats) {
                return stats;
            }
            stats = await this.__head(path, options);
            await this._afterHead(path, stats, options);
            return stats;
        }
        catch (e) {
            const opts = options;
            await this._handleError({
                ...NotReadableError,
                path,
                e,
            }, errors, async (error) => {
                await this._afterHead(path, null, opts, error);
            });
            return null;
        }
    }
    async list(path, options, errors) {
        const dir = this.getDirectory(path);
        return await dir.list(options, errors);
    }
    async ls(path, options, errors) {
        return await this.list(path, options, errors);
    }
    async mkcol(path, options, errors) {
        const dir = this.getDirectory(path);
        return await dir.mkcol(options, errors);
    }
    async move(fromPath, toPath, options, errors) {
        options = { ...this.defaultMoveOptions, ...options };
        let info;
        try {
            info = await this._prepareCopy(fromPath, toPath, options);
        }
        catch (e) {
            if (errors) {
                errors.push(e);
                return false;
            }
            throw e;
        }
        const { from, to } = info;
        return await from.move(to, options, errors);
    }
    async patch(path, props, options, errors) {
        path = this._checkPath(path);
        options = { ...this.defaultPatchOptions, ...options };
        if (path.endsWith("/")) {
            if (!options.type) {
                options.type = EntryType.Directory;
            }
        }
        const stats = await this.head(path, options, errors);
        if (!stats) {
            return false;
        }
        this._fixProps(path, props, stats);
        try {
            const result = await this._beforePatch(path, props, options);
            if (result != null) {
                return result;
            }
            await this._doPatch(path, stats, props, options);
            await this._afterPatch(path, true, options);
            return true;
        }
        catch (e) {
            const opts = options;
            await this._handleError({
                ...NoModificationAllowedError,
                path,
                e,
            }, errors, async (errors) => {
                await this._afterPatch(path, false, opts, errors);
            });
            return false;
        }
    }
    async read(path, type, options, errors) {
        const file = this.getFile(path);
        return await file.read(type, options, errors);
    }
    readdir(path, options, errors) {
        return this.list(path, options, errors);
    }
    async stat(path, options, errors) {
        return await this.head(path, options, errors);
    }
    async write(path, data, options, errors) {
        const file = this.getFile(path);
        return await file.write(data, options, errors);
    }
    async __head(path, options) {
        if (options.type === EntryType.Directory && !this.supportDirectory()) {
            return {};
        }
        const stats = await this._doHead(path, options);
        let errorMessage;
        if (stats.size != null && options.type === EntryType.Directory) {
            errorMessage = `"${path}" is not a directory`;
        }
        if (stats.size == null && options.type === EntryType.File) {
            errorMessage = `"${path}" is not a file`;
        }
        if (errorMessage) {
            throw createError({
                ...TypeMismatchError,
                repository: this.repository,
                path,
                message: errorMessage,
            });
        }
        return stats;
    }
    async _afterHead(path, stats, options, error) {
        const afterHead = this.options.hook?.afterHead;
        if (afterHead && !options.ignoreHook) {
            await afterHead(this.repository, path, options, stats, error);
        }
    }
    async _afterPatch(path, result, options, error) {
        const afterPatch = this.options.hook?.afterPatch;
        if (afterPatch && !options.ignoreHook) {
            await afterPatch(this.repository, path, options, result, error);
        }
    }
    async _beforeHead(path, options) {
        const beforeHead = this.options.hook?.beforeHead;
        if (beforeHead && !options?.ignoreHook) {
            return await beforeHead(this.repository, path, options);
        }
        return null;
    }
    async _beforePatch(path, props, options) {
        const beforePatch = this.options.hook?.beforePatch;
        if (beforePatch && !options.ignoreHook) {
            return await beforePatch(this.repository, path, props, options);
        }
        return null;
    }
    _checkPath(path) {
        if (!path || INVALID_CHARS.test(path)) {
            throw createError({
                name: SyntaxError.name,
                repository: this.repository,
                path,
                message: `"${path}" is invalid`,
            });
        }
        return normalizePath(path);
    }
    _fixProps(path, props, stats) {
        if (props.size != null) {
            console.warn(`Cannot change size: ${path}`);
            delete props.size;
        }
        if (props.etag != null) {
            console.warn(`Cannot change etag: ${path}`);
            delete props.etag;
        }
        if (this.canPatchAccessed()) {
            if (typeof props.accessed !== "number") {
                console.warn(`Access time (${props.accessed}) is illegal: ${path}`);
                delete props.accessed;
            }
        }
        else {
            console.warn(`Cannot patch access time on the FileSystem: ${this.constructor.name}`);
            delete props.accessed;
        }
        if (this.canPatchCreated()) {
            if (typeof props.created !== "number") {
                console.warn(`Creation time (${props.created}) is illegal: ${path}`);
                delete props.created;
            }
        }
        else {
            console.warn(`Cannot patch creation time on the FileSystem: ${this.constructor.name}`);
            delete props.created;
        }
        if (this.canPatchModified()) {
            if (typeof props.modified !== "number") {
                console.warn(`Modification time (${props.modified}) is illegal: ${path}`);
                delete props.modified;
            }
        }
        else {
            console.warn(`Cannot patch modification time on the FileSystem: ${this.constructor.name}`);
            delete props.modified;
        }
        for (const key of Object.keys(stats)) {
            if (stats[key] === props[key]) {
                delete props[key];
            }
            else if (typeof stats[key] !== typeof props[key] &&
                typeof props[key] !== "undefined") {
                console.warn(`Illetal type stats[${key}]: ${props[key]}`);
                delete props[key];
            }
        }
    }
    async _prepareCopy(fromPath, toPath, options) {
        let from;
        try {
            from = await this.getEntry(fromPath, options);
        }
        catch (e) {
            if (isFileSystemError(e) && e.name === NotFoundError.name) {
                if (!this.supportDirectory()) {
                    from = this.getDirectory(fromPath);
                }
                else {
                    throw e;
                }
            }
            else {
                throw e;
            }
        }
        const to = from instanceof AbstractFile
            ? this.getFile(toPath)
            : this.getDirectory(toPath);
        return { from, to };
    }
}
//# sourceMappingURL=AbstractFileSystem.js.map