export class FileSystemError extends Error {
    constructor(params) {
        super(params.message);
        for (const [key, value] of Object.entries(params)) {
            this[key] = value;
        }
    }
    toString() {
        return JSON.stringify(this, null, 2);
    }
}
export const IndexSizeError = {
    code: "1",
    name: "IndexSizeError",
    message: "The index is not in the allowed range.",
};
export const HierarchyRequestError = {
    code: "3",
    name: "HierarchyRequestError",
    message: "The operation would yield an incorrect node tree. [DOM]",
};
export const WrongDocumentError = {
    code: "4",
    name: "WrongDocumentError",
    message: "The object is in the wrong document. [DOM]",
};
export const InvalidCharacterError = {
    code: "5",
    name: "InvalidCharacterError",
    message: "The string contains invalid characters.",
};
export const NoModificationAllowedError = {
    code: "7",
    name: "NoModificationAllowedError",
    message: "The string contains invalid characters.",
};
export const NotFoundError = {
    code: "8",
    name: "NotFoundError",
    message: "The object can not be found here.",
};
export const NotSupportedError = {
    code: "9",
    name: "NotSupportedError",
    message: "The operation is not supported.",
};
export const InUseAttributeError = {
    code: "10",
    name: "InUseAttributeError",
    message: "The attribute is in use.",
};
export const InvalidStateError = {
    code: "11",
    name: "InvalidStateError",
    message: "The object is in an invalid state.",
};
export const SyntaxError = {
    code: "12",
    name: "SyntaxError",
    message: "The string did not match the expected pattern.",
};
export const InvalidModificationError = {
    code: "13",
    name: "InvalidModificationError",
    message: "The object can not be modified in this way.",
};
export const NamespaceError = {
    code: "14",
    name: "NamespaceError",
    message: "The operation is not allowed by Namespaces in XML. [XML-NAMES]",
};
export const InvalidAccessError = {
    code: "15",
    name: "InvalidAccessError",
    message: "The object does not support the operation or argument.",
};
export const TypeMismatchError = {
    code: "17",
    name: "TypeMismatchError",
    message: "The type of the object does not match the expected type.",
};
export const SecurityError = {
    code: "18",
    name: "SecurityError",
    message: "The operation is insecure.",
};
export const NetworkError = {
    code: "19",
    name: "NetworkError",
    message: "A network error occurred.",
};
export const AbortError = {
    code: "20",
    name: "AbortError",
    message: "The operation was aborted.",
};
export const URLMismatchError = {
    code: "21",
    name: "URLMismatchError",
    message: "The given URL does not match another URL.",
};
export const QuotaExceededError = {
    code: "22",
    name: "QuotaExceededError",
    message: "The quota has been exceeded.",
};
export const TimeoutError = {
    code: "23",
    name: "TimeoutError",
    message: "The operation timed out.",
};
export const InvalidNodeTypeError = {
    code: "24",
    name: "InvalidNodeTypeError",
    message: "The supplied node is incorrect or has an incorrect ancestor for this operation.",
};
export const DataCloneError = {
    code: "25",
    name: "DataCloneError",
    message: "The object can not be cloned.",
};
export const EncodingError = {
    name: "EncodingError",
    message: "The encoding operation (either encoded or decoding) failed.",
};
export const NotReadableError = {
    name: "NotReadableError",
    message: "The I/O read operation failed.",
};
export const UnknownError = {
    name: "UnknownError",
    message: "The operation failed for an unknown transient reason.",
};
export const ConstraintError = {
    name: "ConstraintError",
    message: "A mutation operation in a transaction failed because a constraint was not satisfied.",
};
export const DataError = {
    name: "DataError",
    message: "Provided data is inadequate.",
};
export const TransactionInactiveError = {
    name: "TransactionInactiveError",
    message: "A request was placed against a transaction which is currently not active, or which is finished.",
};
export const ReadOnlyError = {
    name: "ReadOnlyError",
    message: 'The mutating operation was attempted in a "readonly" transaction.',
};
export const VersionError = {
    name: "VersionError",
    message: "An attempt was made to open a database using a lower version than the existing version.",
};
export const OperationError = {
    name: "OperationError",
    message: "The operation failed for an operation-specific reason.",
};
export const NotAllowedError = {
    name: "NotAllowedError",
    message: "The request is not allowed by the user agent or the platform in the current context, possibly because the user denied permission.",
};
export const PathExistError = {
    code: "12",
    name: "PathExistError",
    message: "The request file or directry has already existed.",
};
export const domExceptions = [
    IndexSizeError,
    HierarchyRequestError,
    WrongDocumentError,
    InvalidCharacterError,
    NoModificationAllowedError,
    NotFoundError,
    NotSupportedError,
    InUseAttributeError,
    InvalidStateError,
    SyntaxError,
    InvalidModificationError,
    NamespaceError,
    InvalidAccessError,
    TypeMismatchError,
    SecurityError,
    NetworkError,
    AbortError,
    URLMismatchError,
    QuotaExceededError,
    TimeoutError,
    InvalidNodeTypeError,
    DataCloneError,
    EncodingError,
    NotReadableError,
    UnknownError,
    ConstraintError,
    DataError,
    TransactionInactiveError,
    ReadOnlyError,
    VersionError,
    OperationError,
    NotAllowedError,
];
export function createError(error) {
    if (isFileSystemError(error.e)) {
        return error.e;
    }
    let found = false;
    const e = error.e;
    if (e?.name) {
        for (const de of domExceptions) {
            if (de.name === e.name) {
                found = true;
                error.name = de.name;
                error.code = de.code;
                if (!error.message) {
                    error.message = de.message;
                }
                break;
            }
        }
    }
    if (!found) {
        for (const de of domExceptions) {
            if (de.name === error.name) {
                error.name = de.name;
                error.code = de.code;
                if (!error.message) {
                    error.message = de.message;
                }
                break;
            }
        }
    }
    if (!error.name) {
        error.name = UnknownError.name;
        error.message = UnknownError.message;
    }
    return new FileSystemError(error);
}
export function isFileSystemError(e) {
    return e instanceof FileSystemError;
}
//# sourceMappingURL=errors.js.map